@@ -2,7 +2,7 @@
  *
  * Information about ssh channels.
  *
- * $Id: channel.h,v 1.38 1999/06/30 20:43:49 nisse Exp $
+ * $Id: channel.h,v 1.41 1999/08/28 19:57:03 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -60,8 +60,16 @@
      (super flow_controlled)
      (vars
        ; Remote channel number 
-       (channel_number simple UINT32)
+       (channel_number . UINT32)
 
+       ; Somewhat redundant. Makes it easier to locate
+       ; the channel_table entry for the channel, which
+       ; is needed for deallocating it.
+       ;; (local_number . UINT32)
+       
+       ; Where to pass errors
+       (e object exception_handler)
+       
        ; We try to keep the rec_window_size between max_window / 2
        ; and max_window.
        (max_window simple UINT32)       
@@ -93,18 +101,18 @@
        ; propagate a channel broken message to the other end? 
 
        ; Type is CHANNEL_DATA or CHANNEL_STDERR_DATA
-       (receive method int "int type" "struct lsh_string *data")
+       (receive method void "int type" "struct lsh_string *data")
 
        ; Called when we are allowed to send data on the channel. 
-       (send method int)
+       (send method void)
 
        ; Called when the channel is closed
        ; FIXME: Is this needed for anything?
-       (close method int)
+       (close method void)
 
        ; Called when eof is received on the channel (or when it is
        ; closed, whatever happens first).
-       (eof method int)
+       (eof method void)
   
        ; Reply from SSH_MSG_CHANNEL_OPEN_REQUEST
        ;; (open_confirm method int)
@@ -136,13 +144,10 @@
 #define CHANNEL_OPEN_FAILURE(s) \
 ((s)->open_failure((s)))
 
-     
-/* FIXME: Perhaps, this information is better kept in the connection
- * object? */
-
 /* GABA:
    (class
      (name channel_table)
+     ;; (super exception_handler)
      (vars
        ; FIXME: This is relevant only for the server side. It's
        ; probably better to store this in the connection struct.
@@ -150,8 +155,8 @@
        ;; uid_t user;  ; Authenticated user 
 
        ; Channels are indexed by local number
-       (channels pointer (object ssh_channel) used_channels)
-
+       (channels space (object ssh_channel) used_channels)
+       
        ; Global requests that we support
        (global_requests object alist)
        ; Channel types that we can open
@@ -161,6 +166,11 @@
        ; method as is traditionally used for allocation of unix file 
        ; descriptors.
 
+       ; Channel numbers can be reserved before there is any actual channel
+       ; assigned to them. So the channels table is not enough for keeping track of which
+       ; numbers are in use.
+       (in_use space UINT8)
+       
        (allocated_channels simple UINT32)
        (next_channel simple UINT32)
      
@@ -187,36 +197,51 @@
        ))
 */
 
+#define CHANNEL_EXC_HANDLER(c) (&((c)->table.super))
+
 /* SSH_MSG_GLOBAL_REQUEST */
 
 /* GABA:
    (class
      (name global_request_callback)
      (vars
-       (response method int "int success")
+       (response method void "int success")
        (connection object ssh_connection)))
 */
 
 #define GLOBAL_REQUEST_CALLBACK(c, s) \
-((c) ? ((c)->response((c), (s))) : LSH_OK | LSH_GOON)
+((c)->response((c), (s)))
 
 /* GABA:
    (class
      (name global_request)
      (vars
-       (handler method int "struct ssh_connection *connection"
-                           "struct simple_buffer *args"
-			   "struct global_request_callback *response")))
+       (handler method void "struct ssh_connection *connection"
+                            "struct simple_buffer *args"
+			    "struct global_request_callback *response")))
 */
 
 #define GLOBAL_REQUEST(r, c, a, n) ((r)->handler((r), (c), (a), (n)))
 
 /* SSH_MSG_CHANNEL_OPEN */
-  
+
+/* Raised if opening of a channel fails. Used both on the client and
+ * the server side.*/
+/* GABA:
+   (class
+     (name channel_open_exception)
+     (super exception)
+     (vars
+       (error_code . UINT32)))
+*/
+
+struct exception *make_channel_open_exception(UINT32 error_code, char *msg);
+
+#if 0
 /* Callback function, used to report success or failure for a
  * requested channel open. */
   
-/* GABA:
+/* ;;GABA:
    (class
      (name channel_open_callback)
      (vars
@@ -230,26 +255,28 @@
 
 #define CHANNEL_OPEN_CALLBACK(c, ch, e, m, a) \
   (c)->response((c), (ch), (e), (m), (a))
+#endif
 
 /* GABA:
    (class
      (name channel_open)
      (vars
-       (handler method int
+       (handler method void
                 "struct ssh_connection *connection"
                 "struct simple_buffer *data"
-                "struct channel_open_callback *response")))
+                "struct command_continuation *c"
+		"struct exception_handler *e")))
 */
 
-#define CHANNEL_OPEN(o, c, d, n) \
-((o)->handler((o), (c), (d), (n)))
+#define CHANNEL_OPEN(o, c, d, r, e) \
+((o)->handler((o), (c), (d), (r), (e)))
 
 /* SSH_MSG_CHANNEL_REQUEST */
 /* GABA:
    (class
      (name channel_request)
      (vars
-       (handler method int
+       (handler method void
 		"struct ssh_channel *channel"
 		"struct ssh_connection *connection"
 		"int want_reply"
@@ -274,15 +301,19 @@ void init_channel(struct ssh_channel *channel);
 struct channel_table *make_channel_table(void);
 int alloc_channel(struct channel_table *table);
 void dealloc_channel(struct channel_table *table, int i);
-int register_channel(struct channel_table *table, struct ssh_channel *channel);
+
+void register_channel(struct channel_table *table,
+		      UINT32 local_channel_number,
+		      struct ssh_channel *channel);
+
 struct ssh_channel *lookup_channel(struct channel_table *table, UINT32 i);
 
 struct abstract_write *make_channel_write(struct ssh_channel *channel);
 struct abstract_write *make_channel_write_extended(struct ssh_channel *channel,
 						   UINT32 type);
 
-struct read_handler *make_channel_read_data(struct ssh_channel *channel);
-struct read_handler *make_channel_read_stderr(struct ssh_channel *channel);
+struct io_read_callback *make_channel_read_data(struct ssh_channel *channel);
+struct io_read_callback *make_channel_read_stderr(struct ssh_channel *channel);
 
 struct lsh_string *format_global_failure(void);
 struct lsh_string *format_global_success(void);
@@ -299,7 +330,7 @@ struct lsh_string *format_channel_failure(UINT32 channel);
 struct lsh_string *prepare_window_adjust(struct ssh_channel *channel,
 					 UINT32 add);
 
-int channel_start_receive(struct ssh_channel *channel);
+void channel_start_receive(struct ssh_channel *channel);
 
 struct lsh_string *prepare_channel_open(struct channel_table *table,
 					int type,
@@ -314,8 +345,8 @@ struct lsh_string *format_channel_request(int type,
 struct lsh_string *format_channel_close(struct ssh_channel *channel);
 struct lsh_string *format_channel_eof(struct ssh_channel *channel);
 
-int channel_close(struct ssh_channel *channel);
-int channel_eof(struct ssh_channel *channel);
+void channel_close(struct ssh_channel *channel);
+void channel_eof(struct ssh_channel *channel);
 
 struct close_callback *make_channel_close(struct ssh_channel *channel);
 
