@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.67 1999/06/14 01:23:49 nisse Exp $ */
+ * $Id: client.c,v 1.71 1999/08/29 17:43:07 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -61,12 +61,15 @@
      (super packet_handler)
      (vars
        (service simple int)
-       (c object command_continuation)))
+       (c object command_continuation)
+       ;; Do we really need the exception handler here?
+       (e object exception_handler)))
 */
 
-static int do_accept_service(struct packet_handler *c,
-			     struct ssh_connection *connection,
-			     struct lsh_string *packet)
+static void
+do_accept_service(struct packet_handler *c,
+		  struct ssh_connection *connection,
+		  struct lsh_string *packet)
 {
   CAST(accept_service_handler, closure, c);
 
@@ -85,23 +88,29 @@ static int do_accept_service(struct packet_handler *c,
       lsh_string_free(packet);
       connection->dispatch[SSH_MSG_SERVICE_ACCEPT] = connection->fail;
       
-      return COMMAND_RETURN(closure->c, connection);
+      COMMAND_RETURN(closure->c, connection);
+    }
+  else
+    {
+      lsh_string_free(packet);
+      EXCEPTION_RAISE(closure->e,
+		      make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
+					      "Invalid SSH_MSG_SERVICE_ACCEPT message"));
     }
-
-  lsh_string_free(packet);
-  return LSH_FAIL | LSH_DIE;
 }
 
 struct packet_handler *
 make_accept_service_handler(int service,
-			    struct command_continuation *c)
+			    struct command_continuation *c,
+			    struct exception_handler *e)
 {
   NEW(accept_service_handler, closure);
 
   closure->super.handler = do_accept_service;
   closure->service = service;
   closure->c = c;
-
+  closure->e = e;
+  
   return &closure->super;
 }
 
@@ -114,18 +123,20 @@ make_accept_service_handler(int service,
        ;; (service object ssh_service)))
 */
 
-static int do_request_service(struct command *s,
-			      struct lsh_object *x,
-			      struct command_continuation *c)
+static void
+do_request_service(struct command *s,
+		   struct lsh_object *x,
+		   struct command_continuation *c,
+		   struct exception_handler *e)
 {
   CAST(request_service, self, s);
   CAST(ssh_connection, connection, x);
   
   connection->dispatch[SSH_MSG_SERVICE_ACCEPT]
-    = make_accept_service_handler(self->service, c);
+    = make_accept_service_handler(self->service, c, e);
   
-  return A_WRITE(connection->write,
-		 format_service_request(self->service));
+  C_WRITE(connection,
+	  format_service_request(self->service));
 }
 
 struct command *make_request_service(int service)
@@ -179,7 +190,8 @@ struct command *make_request_service(int service)
 */
 
 /* Callback used when the server sends us eof */
-static int do_client_session_eof(struct ssh_channel *c)
+static void
+do_client_session_eof(struct ssh_channel *c)
 {
   CAST(client_session, session, c);
   
@@ -188,13 +200,15 @@ static int do_client_session_eof(struct ssh_channel *c)
   close_fd(&session->out->super, 0);
   close_fd(&session->err->super, 0);
 #endif
-
-  return LSH_OK;
 }  
 
-static int do_client_session_close(struct ssh_channel *c UNUSED)
+static void
+do_client_session_close(struct ssh_channel *c)
 {
-  return LSH_CHANNEL_PENDING_CLOSE;
+  static const struct exception finish_exception
+    = STATIC_EXCEPTION(EXC_FINISH_PENDING, "Session closed.");
+
+  EXCEPTION_RAISE(c->e, &finish_exception);
 }
 
 
@@ -206,11 +220,12 @@ static int do_client_session_close(struct ssh_channel *c UNUSED)
        (exit_status simple "int *")))
 */
 
-static int do_exit_status(struct channel_request *c,
-			  struct ssh_channel *channel,
-			  struct ssh_connection *connection UNUSED,
-			  int want_reply,
-			  struct simple_buffer *args)
+static void
+do_exit_status(struct channel_request *c,
+	       struct ssh_channel *channel,
+	       struct ssh_connection *connection UNUSED,
+	       int want_reply,
+	       struct simple_buffer *args)
 {
   CAST(exit_handler, closure, c);
   int status;
@@ -224,25 +239,27 @@ static int do_exit_status(struct channel_request *c,
       ALIST_SET(channel->request_types, ATOM_EXIT_STATUS, NULL);
       ALIST_SET(channel->request_types, ATOM_EXIT_SIGNAL, NULL);
 
-      /* Sent EOF, if we haven't done that already. */
+      /* Send EOF, if we haven't done that already. */
       /* FIXME: Make this behaviour configurable, there may be some
        * child process alive that we could talk to. */
 
       if (!(channel->flags & CHANNEL_SENT_EOF))
-	return channel_eof(channel);
-      
-      return LSH_OK | LSH_GOON;
+	channel_eof(channel);
     }
-  
-  /* Invalid request */
-  return LSH_FAIL | LSH_DIE;
+  else
+    /* Invalid request */
+    EXCEPTION_RAISE
+      (channel->e,
+       make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
+			       "Invalid exit-status message"));
 }
 
-static int do_exit_signal(struct channel_request *c,
-			  struct ssh_channel *channel,
-			  struct ssh_connection *connection UNUSED,
-			  int want_reply,
-			  struct simple_buffer *args)
+static void
+do_exit_signal(struct channel_request *c,
+	       struct ssh_channel *channel,
+	       struct ssh_connection *connection UNUSED,
+	       int want_reply,
+	       struct simple_buffer *args)
 {
   CAST(exit_handler, closure, c);
 
@@ -282,13 +299,14 @@ static int do_exit_signal(struct channel_request *c,
        * child process alive that we could talk to. */
 
       if (!(channel->flags & CHANNEL_SENT_EOF))
-	return channel_eof(channel);
-
-      return LSH_OK | LSH_GOON;
+	channel_eof(channel);
     }
-  
-  /* Invalid request */
-  return LSH_FAIL | LSH_DIE;
+  else
+    /* Invalid request */
+    EXCEPTION_RAISE
+      (channel->e,
+       make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
+			       "Invalid exit-signal message"));
 }
 
 struct channel_request *make_handle_exit_status(int *exit_status)
@@ -314,46 +332,52 @@ struct channel_request *make_handle_exit_signal(int *exit_status)
 }
 
 /* Receive channel data */
-static int do_receive(struct ssh_channel *c,
-		      int type, struct lsh_string *data)
+static void
+do_receive(struct ssh_channel *c,
+	   int type, struct lsh_string *data)
 {
   CAST(client_session, closure, c);
   
   switch(type)
     {
     case CHANNEL_DATA:
-      return A_WRITE(&closure->out->buffer->super, data);
+      A_WRITE(&closure->out->write_buffer->super, data, c->e);
+      break;
     case CHANNEL_STDERR_DATA:
-      return A_WRITE(&closure->err->buffer->super, data);
+      A_WRITE(&closure->err->write_buffer->super, data, c->e);
+      break;
     default:
       fatal("Internal error!\n");
     }
 }
 
 /* We may send more data */
-static int do_send(struct ssh_channel *c)
+static void do_send(struct ssh_channel *c)
 {
   CAST(client_session, closure, c);
 
   assert(closure->in->super.read);
-  assert(closure->in->handler);
-  closure->in->super.want_read = 1;
 
-  return LSH_OK | LSH_GOON;
+  closure->in->super.want_read = 1;
 }
 
 /* We have a remote shell */
-static int do_client_io(struct command *s UNUSED,
-			struct lsh_object *x,
-			struct command_continuation *c)
+static void
+do_client_io(struct command *s UNUSED,
+	     struct lsh_object *x,
+	     struct command_continuation *c,
+	     struct exception_handler *e UNUSED)
 
 {
+  assert(x);
+#if 0
   if (!x)
     {
       werror("do_client_io: Starting shell failed.\n");
       return LSH_CHANNEL_CLOSE | LSH_FAIL;
     }
   else
+#endif
     {
       CAST(client_session, session, x);
       struct ssh_channel *channel = &session->super;
@@ -363,7 +387,7 @@ static int do_client_io(struct command *s UNUSED,
       session->out->super.close_callback
 	= session->err->super.close_callback = make_channel_close(channel);
   
-      session->in->handler = make_channel_read_data(channel);
+      session->in->super.read = make_channel_read_data(channel);
       channel->send = do_send;
 
       ALIST_SET(channel->request_types, ATOM_EXIT_STATUS,
@@ -373,7 +397,7 @@ static int do_client_io(struct command *s UNUSED,
 
       channel->eof = do_client_session_eof;
       
-      return COMMAND_RETURN(c, channel);
+      COMMAND_RETURN(c, channel);
     }
 }
 
@@ -409,8 +433,8 @@ struct ssh_channel *make_client_session(struct io_fd *in,
   self->err = err;
 
   /* Flow control */
-  out->buffer->report = &self->super.super;
-  err->buffer->report = &self->super.super;
+  out->write_buffer->report = &self->super.super;
+  err->write_buffer->report = &self->super.super;
   
   self->exit_status = exit_status;
   
@@ -437,7 +461,7 @@ new_session(struct channel_open_command *s,
 
   self->session->write = connection->write;
   
-  *request = prepare_channel_open(connection->channels, ATOM_SESSION,
+  *request = prepare_channel_open(connection->table, ATOM_SESSION,
 				  self->session, "");
   if (!*request)
     return NULL;
