@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: client.h,v 1.23 1999/04/25 15:16:03 nisse Exp $ */
+ * $Id: client.h,v 1.24 1999/07/10 08:56:16 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -45,7 +45,8 @@ struct close_callback *make_client_close_handler(void);
 
 struct packet_handler *
 make_accept_service_handler(int service,
-			    struct command_continuation *c);
+			    struct command_continuation *c,
+			    struct exception_handler *e);
 
 struct command *make_request_service(int service);
 
