@@ -2,7 +2,7 @@
  *
  * Builtin combinator functions (S, K, ...)
  *
- * $Id: combinators.c,v 1.5 1999/06/22 22:48:22 nisse Exp $ */
+ * $Id: combinators.c,v 1.7 1999/08/22 21:24:02 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -104,36 +104,44 @@ struct command_simple command_K = STATIC_COMMAND_SIMPLE(do_simple_command_K);
        (x object lsh_object)))
 */
 
-static int do_command_S_continuation(struct command_continuation *c,
-				     struct lsh_object *value)
+static void
+do_command_S_continuation(struct command_continuation *c,
+			  struct lsh_object *value)
 {
   CAST(command_S_continuation, self, c);
   CAST_SUBTYPE(command, op, value);
-  return COMMAND_CALL(self->g, self->x, make_apply(op, self->super.up));
+  COMMAND_CALL(self->g, self->x,
+	       make_apply(op, self->super.up, self->super.e),
+	       self->super.e);
 }
 
 static struct command_continuation *
 make_command_S_continuation(struct command *g,
 			    struct lsh_object *x,
-			    struct command_continuation *up)
+			    struct command_continuation *up,
+			    struct exception_handler *e)
 {
   NEW(command_S_continuation, c);
   c->g = g;
   c->x = x;
   c->super.up = up;
+  c->super.e = e;
   c->super.super.c = do_command_S_continuation;
 
   return &c->super.super;
 }
 
-static int do_command_S_2(struct command *s,
-			  struct lsh_object *x,
-			  struct command_continuation *c)
+static void
+do_command_S_2(struct command *s,
+	       struct lsh_object *x,
+	       struct command_continuation *c,
+	       struct exception_handler *e)
 {
   CAST(command_S_2, self, s);
   
-  return COMMAND_CALL(self->f, x,
-		      make_command_S_continuation(self->g, x, c));
+  COMMAND_CALL(self->f, x,
+	       make_command_S_continuation(self->g, x, c, e),
+	       e);
 }
 
 static struct lsh_object *
@@ -190,15 +198,19 @@ STATIC_COLLECT_1(&collect_info_S_2);
        (g object command)))
 */
 
-static int do_command_Sp_3(struct command *s,
-			   struct lsh_object *x,
-			   struct command_continuation *c)
+static void
+do_command_Sp_3(struct command *s,
+		struct lsh_object *x,
+		struct command_continuation *c,
+		struct exception_handler *e)
 {
   CAST(command_Sp_3, self, s);
-  return COMMAND_CALL(self->f, x,
-		      make_apply(self->c,
-				 make_command_S_continuation(self->g,
-							     x, c)));
+  COMMAND_CALL(self->f, x,
+	       make_apply(self->c,
+			  make_command_S_continuation(self->g,
+						      x, c, e),
+			  e),
+	       e);
 }
 
 static struct lsh_object *
@@ -262,12 +274,15 @@ STATIC_COLLECT_1(&collect_info_Sp_2);
        (g object command)))
 */
 
-static int do_command_B_2(struct command *s,
-			  struct lsh_object *x,
-			  struct command_continuation *c)
+static void
+do_command_B_2(struct command *s,
+	       struct lsh_object *x,
+	       struct command_continuation *c,
+	       struct exception_handler *e)
 {
   CAST(command_B_2, self, s);
-  return COMMAND_CALL(self->g, x, make_apply(self->f, c));
+  COMMAND_CALL(self->g, x,
+	       make_apply(self->f, c, e), e);
 }
 
 static struct lsh_object *
@@ -324,19 +339,22 @@ STATIC_COLLECT_1(&collect_info_B_2);
        (g object command)))
 */
 
-static int do_command_Bp_3(struct command *s,
-			  struct lsh_object *x,
-			  struct command_continuation *c)
+static void
+do_command_Bp_3(struct command *s,
+		struct lsh_object *x,
+		struct command_continuation *c,
+		struct exception_handler *e)
 {
   CAST(command_Bp_3, self, s);
-  return COMMAND_CALL(self->g, x,
-		      make_apply(self->f,
-				 make_apply(self->c, c)));
+  COMMAND_CALL(self->g, x,
+	       make_apply(self->f,
+			  make_apply(self->c, c, e), e),
+	       e);
 }
 
 static struct lsh_object *
 do_simple_command_Bp_3(struct command_simple *s,
-		      struct lsh_object *x)
+		       struct lsh_object *x)
 {
   CAST(command_Bp_3, self, s);
   CAST_SUBTYPE(command_simple, cs, self->c);
@@ -403,34 +421,41 @@ STATIC_COLLECT_1(&collect_info_Bp_2);
        (y object lsh_object)))
 */
 
-static int do_command_C_continuation(struct command_continuation *c,
-				     struct lsh_object *value)
+static void
+do_command_C_continuation(struct command_continuation *c,
+			  struct lsh_object *value)
 {
   CAST(command_C_continuation, self, c);
   CAST_SUBTYPE(command, op, value);
-  return COMMAND_CALL(op, self->y, self->super.up);
+  
+  COMMAND_CALL(op, self->y, self->super.up, self->super.e);
 }
 
 static struct command_continuation *
 make_command_C_continuation(struct lsh_object *y,
-			    struct command_continuation *up)
+			    struct command_continuation *up,
+			    struct exception_handler *e)
 {
   NEW(command_C_continuation, c);
   c->y = y;
   c->super.up = up;
+  c->super.e = e;
   c->super.super.c = do_command_C_continuation;
   
   return &c->super.super;
 }
 
-static int do_command_C_2(struct command *s,
-			  struct lsh_object *x,
-			  struct command_continuation *c)
+static void
+do_command_C_2(struct command *s,
+	       struct lsh_object *x,
+	       struct command_continuation *c,
+	       struct exception_handler *e)
 {
   CAST(command_C_2, self, s);
   
-  return COMMAND_CALL(self->f, x,
-		      make_command_C_continuation(self->y, c));
+  COMMAND_CALL(self->f, x,
+	       make_command_C_continuation(self->y, c, e),
+	       e);
 }
 
 static struct lsh_object *do_simple_command_C_2(struct command_simple *s,
@@ -484,15 +509,18 @@ STATIC_COLLECT_1(&collect_info_C_2);
        (y object lsh_object)))
 */
 
-static int do_command_Cp_3(struct command *s,
-			  struct lsh_object *x,
-			  struct command_continuation *c)
+static void
+do_command_Cp_3(struct command *s,
+		struct lsh_object *x,
+		struct command_continuation *c,
+		struct exception_handler *e)
 {
   CAST(command_Cp_3, self, s);
   
-  return COMMAND_CALL(self->f, x,
-		      make_apply(self->c,
-				 make_command_C_continuation(self->y, c)));
+  COMMAND_CALL(self->f, x,
+	       make_apply(self->c,
+			  make_command_C_continuation(self->y, c, e), e),
+	       e);
 }
 
 static struct lsh_object *
