@@ -1,6 +1,6 @@
 /* command.h
  *
- * $Id: command.h,v 1.29 1999/06/30 20:54:27 nisse Exp $ */
+ * $Id: command.h,v 1.33 1999/08/28 19:57:05 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -26,6 +26,7 @@
 
 #include "lsh.h"
 
+#include "exception.h"
 #include "list.h"
 #include "io.h"
 
@@ -42,22 +43,16 @@
    (class
      (name command_continuation)
      (vars
-       (c method int "struct lsh_object *result")))
-*/
-
-/* GABA:
-   (class
-     (name command_exception)
-     (vars
-       (e method int UINT32)))
+       (c method void "struct lsh_object *result")))
 */
 
 /* GABA:
    (class
      (name command)
      (vars
-       (call method int "struct lsh_object *arg"
-                        "struct command_continuation *c")))
+       (call method void "struct lsh_object *arg"
+                         "struct command_continuation *c"
+			 "struct exception_handler *e")))
 */
 
 /* GABA:
@@ -70,19 +65,18 @@
        (call_simple method "struct lsh_object *" "struct lsh_object *")))
 */
 
-#define COMMAND_CALL(f, a, c) \
-  ((f)->call((f), (struct lsh_object *) (a), (c)))
+#define COMMAND_CALL(f, a, c, e) \
+  ((f)->call((f), (struct lsh_object *) (a), (c), (e)))
+
+#define COMMAND_RETURN(r, v) ((r)->c((r), (struct lsh_object *) (v)))
 
-/* NOTE: If R == NULL, don't call the continuation, _but_ the argument
- * V must still be evaluated. */
-#define COMMAND_RETURN(r, v) \
-  ((r) ? ((r)->c((r), (struct lsh_object *) (v))) : ((void)(v), LSH_OK | LSH_GOON))
 #define COMMAND_SIMPLE(f, a) \
   ((f)->call_simple((f), (struct lsh_object *)(a)))
 
-int do_call_simple_command(struct command *s,
-			   struct lsh_object *arg,
-			   struct command_continuation *c);
+void do_call_simple_command(struct command *s,
+			    struct lsh_object *arg,
+			    struct command_continuation *c,
+			    struct exception_handler *e);
 
 #define STATIC_COMMAND_SIMPLE(f) \
 { { STATIC_HEADER, do_call_simple_command }, f}
@@ -94,14 +88,35 @@ extern struct command_simple progn_command;
 
 extern struct command_continuation discard_continuation;
 
+#define CONTINUATION_USED_P(c) ((c) != &discard_continuation)
+
 /* GABA:
    (class
      (name command_frame)
      (super command_continuation)
      (vars
-       (up object command_continuation)))
+       (up object command_continuation)
+       (e object exception_handler)))
 */
 
+/* Used when the execution context must be saved for later use.
+ *
+ * FIXME: Perhaps use a struct instead? This pair is needed in many
+ * places.
+ * */
+/* GABA:
+   (class
+     (name command_context)
+     (vars
+       (c object command_continuation)
+       (e object exception_handler)))
+*/
+
+struct command_context *
+make_command_context(struct command_continuation *c,
+		     struct exception_handler *e);
+
+			    
 /* Commands that need to collect some arguments before actually doing
  * anything. */
 
@@ -204,10 +219,43 @@ struct lsh_object *collect_trace(const char *name, struct lsh_object *real);
 #define MAKE_TRACE(name, real) (real)
 #endif
 
+struct command_continuation *
+make_once_continution(const char *msg,
+		      struct command_continuation *up);
+
+/* extern struct command once_command; */
+
+
+/* Delayed application. This is just a convenient way to record how to
+ * apply a function, at some later time. Currently used only by
+ * server_userauth.c. */
+
+/* GABA:
+   (class
+     (name delayed_apply)
+     (super command_context)
+     (vars
+       (f object command)
+       (a object lsh_object)))
+*/
+
+struct delayed_apply *
+make_delayed_apply(struct command *f,
+		   struct lsh_object *a);
+
+#define FORCE_APPLY(d, c, e) COMMAND_CALL((d)->f, (d)->a, (c), (e))
+
+struct command_continuation *
+make_delay_continuation(struct command *f,
+			struct command_continuation *c);
+
+
 /* The GABA_* macros are used by automatically generated evaluation code */
 
 struct command_continuation *
-make_apply(struct command *f, struct command_continuation *c);  
+make_apply(struct command *f,
+	   struct command_continuation *c, struct exception_handler *e);
+
 struct lsh_object *gaba_apply(struct lsh_object *f,
 			      struct lsh_object *x);
 
