@@ -25,8 +25,10 @@
 
 #include "debug.h"
 #include "encrypt.h"
+#include "exception.h"
 #include "format.h"
 #include "disconnect.h"
+#include "io.h"
 #include "keyexchange.h"
 #include "compress.h"
 #include "packet_ignore.h"
@@ -39,12 +41,16 @@
 #include "connection.h.x"
 #undef GABA_DEFINE
 
+#include "connection.c.x"
+
 static const char *packet_types[0x100] =
 #include "packet_types.h"
 ;
 
-static int handle_connection(struct abstract_write *w,
-			     struct lsh_string *packet)
+static void
+handle_connection(struct abstract_write *w,
+		  struct lsh_string *packet,
+		  struct exception_handler *ignored UNUSED)
 {
   CAST(ssh_connection, closure, w);
   UINT8 msg;
@@ -52,7 +58,10 @@ static int handle_connection(struct abstract_write *w,
   if (!packet->length)
     {
       werror("connection.c: Received empty packet!\n");
-      return LSH_FAIL | LSH_DIE;
+      EXCEPTION_RAISE(closure->e,
+		      make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
+					      "Received empty packet."));
+      return;
     }
 
   msg = packet->data[0];
@@ -66,8 +75,10 @@ static int handle_connection(struct abstract_write *w,
       if (msg == SSH_MSG_NEWKEYS)
 	{
 	  werror("Unexpected NEWKEYS message!\n");
-	  lsh_string_free(packet);
-	  return LSH_FAIL | LSH_DIE;
+	  EXCEPTION_RAISE(closure->e,
+			  make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
+						  "Unexpected NEWKEYS message!"));
+	  return;
 	}
       break;
     case KEX_STATE_IGNORE:
@@ -79,15 +90,17 @@ static int handle_connection(struct abstract_write *w,
        * KEXDH_INIT or KEXDH_REPLY message. */
       closure->kex_state = KEX_STATE_IN_PROGRESS;
       lsh_string_free(packet);
-      return LSH_OK | LSH_GOON;
+      return;
 
     case KEX_STATE_IN_PROGRESS:
       if ( (msg == SSH_MSG_NEWKEYS)
 	   || (msg == SSH_MSG_KEXINIT))
 	{
 	  werror("Unexpected KEXINIT or NEWKEYS message!\n");
-	  lsh_string_free(packet);
-	  return LSH_FAIL | LSH_DIE;
+	  EXCEPTION_RAISE(closure->e,
+			  make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
+						  "Unexpected KEXINIT or NEWKEYS message!"));
+	  return;
 	}
       break;
     case KEX_STATE_NEWKEYS:
@@ -96,25 +109,31 @@ static int handle_connection(struct abstract_write *w,
 	{
 	  werror("Expected NEWKEYS message, but received message %i!\n",
 		 msg);
-	  lsh_string_free(packet);
-	  return LSH_FAIL | LSH_DIE;
+	  EXCEPTION_RAISE(closure->e,
+			  make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
+						  "Expected NEWKEYS message"));
+	  return;
 	}
       break;
     default:
       fatal("handle_connection: Internal error.\n");
     }
 
-  return HANDLE_PACKET(closure->dispatch[msg], closure, packet);
+  HANDLE_PACKET(closure->dispatch[msg], closure, packet);
 }
 
-static int do_fail(struct packet_handler *closure,
-		   struct ssh_connection *connection UNUSED,
-		   struct lsh_string *packet)
+static void
+do_fail(struct packet_handler *closure,
+	struct ssh_connection *connection,
+	struct lsh_string *packet)
 {
   CHECK_TYPE(packet_handler, closure);
 
   lsh_string_free(packet);
-  return LSH_FAIL | LSH_DIE;
+
+  EXCEPTION_RAISE
+    (connection->e,
+     make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR, NULL));
 }
 
 struct packet_handler *make_fail_handler(void)
@@ -125,22 +144,22 @@ struct packet_handler *make_fail_handler(void)
   return res;
 }
 
-static int do_unimplemented(struct packet_handler *closure,
-			    struct ssh_connection *connection,
-			    struct lsh_string *packet)
+static void
+do_unimplemented(struct packet_handler *closure,
+		 struct ssh_connection *connection,
+		 struct lsh_string *packet)
 {
-  int res;
   CHECK_TYPE(packet_handler, closure);
 
-  res =  A_WRITE(connection->write,
-		 ssh_format("%c%i",
-			    SSH_MSG_UNIMPLEMENTED,
-			    packet->sequence_number));
-  verbose("Received packet of unimplemented type %i.\n",
-	  packet->data[0]);
+  werror("Received packet of unimplemented type %i.\n",
+	 packet->data[0]);
+
+  C_WRITE(connection,
+	  ssh_format("%c%i",
+		     SSH_MSG_UNIMPLEMENTED,
+		     packet->sequence_number));
   
   lsh_string_free(packet);
-  return res;
 }
 
 struct packet_handler *make_unimplemented_handler(void)
@@ -151,6 +170,48 @@ struct packet_handler *make_unimplemented_handler(void)
   return res;
 }
 
+/* GABA:
+   (class
+     (name exc_protocol_handler)
+     (super exception_handler)
+     (vars
+       (connection object ssh_connection)))
+*/
+
+static void
+do_exc_protocol_handler(struct exception_handler *s,
+			const struct exception *e)
+{
+  CAST(exc_protocol_handler, self, s);
+
+  switch (e->type)
+    {
+    case EXC_PROTOCOL:
+      {
+	CAST_SUBTYPE(protocol_exception, exc, e);
+	if (exc->reason)
+	  C_WRITE(self->connection, format_disconnect(exc->reason, exc->super.msg, ""));
+	
+	EXCEPTION_RAISE(self->super.parent, &finish_read_exception);
+      }
+      break;
+    default:
+      EXCEPTION_RAISE(self->super.parent, e);
+    }
+}
+
+struct exception_handler *
+make_exc_protocol_handler(struct ssh_connection *connection,
+			  struct exception_handler *parent)
+{
+  NEW(exc_protocol_handler, self);
+  self->connection = connection;
+  self->super.parent = parent;
+  self->super.raise = do_exc_protocol_handler;
+
+  return &self->super;
+}
+
 struct ssh_connection *make_ssh_connection(struct command_continuation *c)
 {
   int i;
@@ -235,7 +296,8 @@ struct ssh_connection *make_ssh_connection(struct command_continuation *c)
 
 void connection_init_io(struct ssh_connection *connection,
 			struct abstract_write *raw,
-			struct randomness *r)
+			struct randomness *r,
+			struct exception_handler *e)
 {
   /* Initialize i/o hooks */
   connection->raw = raw;
@@ -247,6 +309,10 @@ void connection_init_io(struct ssh_connection *connection,
 		        connection
 		      );
 
+  /* Exception handler that sends a proper disconnect message on protocol errors */
+  connection->e = make_exc_protocol_handler(connection, e);
+
+  /* Initial encryption state */
   connection->send_crypto = connection->rec_crypto = NULL;
   connection->send_mac = connection->rec_mac = NULL;
   connection->send_compress = connection->rec_compress = NULL;
