@@ -1,6 +1,6 @@
 /* connection_commands.c
  *
- * $Id: connection_commands.c,v 1.3 1999/04/04 21:37:12 nisse Exp $
+ * $Id: connection_commands.c,v 1.10 1999/08/28 19:57:08 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -84,10 +84,12 @@ struct close_callback *make_connection_close_handler(struct ssh_connection *c)
        (fallback object ssh1_fallback)))
 */
 
-static int do_line(struct line_handler **h,
-		   struct read_handler **r,
-		   UINT32 length,
-		   UINT8 *line)
+static void
+do_line(struct line_handler **h,
+	struct read_handler **r,
+	UINT32 length,
+	UINT8 *line,
+	struct exception_handler *e)
 {
   CAST(connection_line_handler, closure, *h);
   
@@ -105,22 +107,21 @@ static int do_line(struct line_handler **h,
 #if WITH_SSH1_FALLBACK
 	  if (closure->fallback)
 	    {
-	      int res;
-	      
 	      assert(closure->mode == CONNECTION_SERVER);
 	      
 	      /* Sending keyexchange packet was delayed. Do it now */
-	      res = initiate_keyexchange(closure->connection,
-					 closure->mode);
+	      initiate_keyexchange(closure->connection,
+				   closure->mode);
 	      
+#if 0
 	      if (LSH_CLOSEDP(res))
 		{
 		  werror("server.c: do_line: "
 			 "Delayed initiate_keyexchange() failed.\n");
 		  *h = NULL;
-		  
-		  return res | LSH_DIE;
+		  return;
 		}
+#endif
 	    }
 #endif /* WITH_SSH1_FALLBACK */
 	  new = 
@@ -143,7 +144,7 @@ static int do_line(struct line_handler **h,
 		  closure->connection->versions[CONNECTION_SERVER]);
 
 	  *r = new;
-	  return LSH_OK | LSH_GOON;
+	  return;
 	}
 #if WITH_SSH1_FALLBACK      
       else if (closure->fallback
@@ -151,9 +152,12 @@ static int do_line(struct line_handler **h,
 	       && !memcmp(line + 4, "1.", 2))
 	{
 	  *h = NULL;
-	  return SSH1_FALLBACK(closure->fallback,
-			       closure->fd,
-			       length, line);
+	  SSH1_FALLBACK(closure->fallback,
+			closure->fd,
+			length, line,
+			e);
+
+	  return;
 	}
 #endif /* WITH_SSH1_FALLBACK */
       else
@@ -164,8 +168,12 @@ static int do_line(struct line_handler **h,
 	  /* FIXME: Clean up properly */
 	  KILL(closure);
 	  *h = NULL;
-	  
-	  return LSH_FAIL | LSH_DIE;
+
+	  EXCEPTION_RAISE(closure->connection->e,
+			  make_protocol_exception
+			  (SSH_DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED,
+			   NULL));
+	  return;
 	}
     }
   else
@@ -174,7 +182,6 @@ static int do_line(struct line_handler **h,
       werror("%ps\n", length, line);
 
       /* Read next line */
-      return LSH_OK | LSH_GOON;
     }
 }
 
@@ -190,7 +197,7 @@ make_connection_read_line(struct ssh_connection *connection, int mode,
   closure->mode = mode;
   closure->fd = fd;
   closure->fallback = fallback;
-  return make_read_line(&closure->super);
+  return make_read_line(&closure->super, connection->e);
 }
 
 /* Takes a fd as argument, and returns a connection object. Never
@@ -214,14 +221,18 @@ make_connection_read_line(struct ssh_connection *connection, int mode,
        (fallback object ssh1_fallback)))
 */
 
-static int do_connection(struct command *s,
-			 struct lsh_object *x,
-			 struct command_continuation *c)
+/* Buffer size when reading from the socket */
+#define BUF_SIZE (1<<14)
+
+static void
+do_connection(struct command *s,
+	      struct lsh_object *x,
+	      struct command_continuation *c,
+	      struct exception_handler *e)
 {
   CAST(connection_command, self, s);
   CAST(io_fd, fd, x);
   struct lsh_string *version;
-  int res;
   
   struct ssh_connection *connection = make_ssh_connection(c);
 
@@ -255,15 +266,26 @@ static int do_connection(struct command *s,
       fatal("do_connection: Internal error\n");
     }
 
+  /* Installing the right exception handler is a little tricky. The
+   * passed in handler is typically the top-level handler provided by
+   * lsh.c or lshd.c. On top of this, we add the io_exception_handler
+   * which takes care of EXC_FINISH_READ exceptions and closes the
+   * connection's socket. And on top of this, we have a
+   * connection_exception handler, which takes care of EXC_PROTOCOL
+   * exceptions, sends a disconnect message, and then raises an
+   * EXC_FINISH_READ exception. */
   connection_init_io
     (connection, 
      &io_read_write(fd,
-		    make_connection_read_line(connection, self->mode,
-					      fd->super.fd, self->fallback),
+		    make_buffered_read
+		    (BUF_SIZE,
+		     make_connection_read_line(connection, self->mode,
+					       fd->super.fd, self->fallback)),
 		    self->block_size,
 		    make_connection_close_handler(connection))
-     ->buffer->super,
-     self->random);
+     ->write_buffer->super,
+     self->random,
+     make_exc_finish_read_handler(&fd->super, e));
 
   connection->versions[self->mode] = version;
   connection->kexinits[self->mode] = MAKE_KEXINIT(self->init); 
@@ -277,16 +299,19 @@ static int do_connection(struct command *s,
    * Furthermore, it should not send any data after the identification string,
    * until the client's identification string is received. */
   if (self->fallback)
-    return A_WRITE(connection->raw,
-		   ssh_format("%lS\n", version));
+    {
+      A_WRITE(connection->raw,
+	      ssh_format("%lS\n", version),
+	      connection->e);
+      return;
+    }
 #endif /* WITH_SSH1_FALLBACK */
 
-  res = A_WRITE(connection->raw,
-		ssh_format("%lS\r\n", version));
-  if (LSH_CLOSEDP(res))
-    return res;
+  A_WRITE(connection->raw,
+	  ssh_format("%lS\r\n", version),
+	  connection->e);
   
-  return res | initiate_keyexchange(connection, self->mode);
+  initiate_keyexchange(connection, self->mode);
 }
 
 struct command *
@@ -310,3 +335,46 @@ make_handshake_command(int mode,
   self->super.call = do_connection;
   return &self->super;
 }
+
+/* GABA:
+   (class
+     (name connection_remember_command)
+     (super command)
+     (vars
+       (connection object ssh_connection)))
+*/
+
+#if 0
+static int do_connection_remember(struct command *s,
+				  struct lsh_object *x,
+				  struct command_continuation *c)
+{
+  CAST(connection_remember_command, self, s);
+  CAST_SUBTYPE(resource, resource, x);
+
+  if (resource)
+    REMEMBER_RESOURCE(self->connection->resources, resource);
+
+  return COMMAND_RETURN(c, resource);
+}
+
+static struct lsh_object *
+collect_connection_remember(struct collect_info_1 *info,
+			     struct lsh_object *x)
+{
+  CAST(ssh_connection, connection, x);
+  NEW(connection_remember_command, self);
+
+  assert(!info->next);
+
+  self->super.call = do_connection_remember;
+  self->connection = connection;
+
+  return &self->super.super;
+}
+
+struct command connection_remember_command =
+STATIC_COMMAND(collect_connection_remember);
+
+#endif
+
