@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.63 1999/06/14 00:30:09 nisse Exp $ */
+ * $Id: io.c,v 1.70 1999/08/29 17:51:21 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -241,14 +241,16 @@ int io_iter(struct io_backend *b)
   return 1;
 }
 
-/* GABA:
+
+/* ;;GABA:
    (class
      (name fd_read)
      (super abstract_read)
        (vars
-         (fd simple int)))
+         (fd object lsh_fd)))
 */
 
+#if 0
 static int do_read(struct abstract_read **r, UINT32 length, UINT8 *buffer)
 {
   CAST(fd_read, closure, *r);
@@ -258,13 +260,17 @@ static int do_read(struct abstract_read **r, UINT32 length, UINT8 *buffer)
       werror("io.c: do_read(): Zero length read was requested.\n");
       return 0;
     }
-    
+
   for (;;)
     {
-      int res = read(closure->fd, buffer, length);
-      if (!res)
+      int res;
+
+      if (! (closure->fd->super.alive && closure->fd->want_read) )
+	return 0;
+      
+      if (! (res = read(closure->fd->fd, buffer, length)) )
 	{
-	  debug("Read EOF on fd %i.\n", closure->fd);
+	  debug("Read EOF on fd %i.\n", closure->fd->fd);
 	  return A_EOF;
 	}
       if (res > 0)
@@ -289,6 +295,162 @@ static int do_read(struct abstract_read **r, UINT32 length, UINT8 *buffer)
 	}
     }
 }
+#endif
+
+static void do_buffered_read(struct io_read_callback *s,
+			     struct lsh_fd *fd)
+{
+  CAST(io_buffered_read, self, s);
+  UINT8 *buffer = alloca(self->buffer_size);
+  int res = read(fd->fd, buffer, self->buffer_size);
+
+  if (res < 0)
+    switch(errno)
+      {
+      case EINTR:
+	break;
+      case EWOULDBLOCK:
+	werror("io.c: read_callback: Unexpected EWOULDBLOCK\n");
+	break;
+      case EPIPE:
+	fatal("Unexpected EPIPE.\n");
+      default:
+	EXCEPTION_RAISE(fd->e, 
+			make_io_exception(EXC_IO_READ, fd,
+					  errno, NULL));
+	break;
+      }
+  else if (res > 0)
+    {
+      UINT32 left = res;
+    
+      while (fd->super.alive && fd->read && left)
+	{
+	  UINT32 done;
+
+	  /* FIXME: What to do if want_read is false? */
+	  assert(fd->want_read);
+	  assert(self->handler);
+
+	  /* NOTE: This call may replace self->handler */
+	  done = READ_HANDLER(self->handler, left, buffer);
+
+	  buffer += done;
+	  left -= done;
+	}
+
+      if (left)
+	verbose("read_buffered(): fd died, %i buffered bytes discarded\n",
+		left);
+    }
+  else
+    EXCEPTION_RAISE(fd->e,
+		    make_io_exception(EXC_IO_EOF, fd, 0, "EOF")) ;
+}
+
+struct io_read_callback *
+make_buffered_read(UINT32 buffer_size,
+		   struct read_handler *handler)
+{
+  NEW(io_buffered_read, self);
+
+  self->super.read = do_buffered_read;
+  self->buffer_size = buffer_size;
+  self->handler = handler;
+
+  return &self->super;
+}
+
+static void do_consuming_read(struct io_read_callback *c,
+			      struct lsh_fd *fd)
+{
+  CAST_SUBTYPE(io_consuming_read, self, c);
+  UINT32 wanted = READ_QUERY(self);
+
+  if (!wanted)
+    {
+      fd->want_read = 0;
+    }
+  else
+    {
+      struct lsh_string *s = lsh_string_alloc(wanted);
+      int res = read(fd->fd, s->data, wanted);
+
+      if (res < 0)
+	switch(errno)
+	  {
+	  case EINTR:
+	    break;
+	  case EWOULDBLOCK:
+	    werror("io.c: read_consume: Unexpected EWOULDBLOCK\n");
+	    break;
+	  case EPIPE:
+	    fatal("io.c: read_consume: Unexpected EPIPE.\n");
+	  default:
+	    EXCEPTION_RAISE(fd->e, 
+			    make_io_exception(EXC_IO_READ,
+					      fd, errno, NULL));
+	    break;
+	  }
+      else if (res > 0)
+	{
+	  s->length = res;
+	  A_WRITE(self->consumer, s, fd->e);
+	}
+      else
+	EXCEPTION_RAISE(fd->e, make_io_exception(EXC_IO_EOF, fd, 0, "EOF")) ;
+    }
+}
+
+/* NOTE: Doesn't initialize the query field. That should be done in
+ * the subclass's constructor. */
+void init_consuming_read(struct io_consuming_read *self,
+			 struct abstract_write *consumer)
+{
+  self->super.read = do_consuming_read;
+  self->consumer = consumer;
+}
+			 
+#if 0
+static void read_callback(struct lsh_fd *fd)
+{
+  CAST(io_fd, self, fd);
+  int res;
+  
+  assert(self->read_buffer);
+
+  res = read_buffer_fill_fd(self->read_buffer_fill_fd, fd->fd);
+
+  if (res < 0)
+    switch(errno)
+      {
+      case EINTR:
+	return;
+      case EWOULDBLOCK:
+	werror("io.c: read_callback: Unexpected EWOULDBLOCK\n");
+      case EPIPE:
+	fatal("EPIPE should probably be handled as EOF. Not implemented.\n");
+      default:
+	EXCEPTION_RAISE(fd->e, fd,
+			make_io_exception(EXC_IO_READ),
+			errno, NULL);
+      }
+
+  
+  while (fd->super.alive && fd->want_read)
+    {
+      if (!read_buffer_flush())
+	{
+	  EXCEPTION_RAISE(fd->e, make_io_exception(EXC_IO_EOF, fd, 0, "EOF"));
+	  break;
+	}
+    }
+
+  read_buffer_justify(self->read_buffer);
+
+  assert(!(self->read_buffer->start
+	   && fd->super.alive && fd->want_read));
+}
 
 static void read_callback(struct lsh_fd *fd)
 {
@@ -345,6 +507,7 @@ static void read_callback(struct lsh_fd *fd)
 	= LSH_FAILUREP(res) ? CLOSE_PROTOCOL_FAILURE : CLOSE_EOF;
     }
 }
+#endif
 
 static void write_callback(struct lsh_fd *fd)
 {
@@ -352,12 +515,12 @@ static void write_callback(struct lsh_fd *fd)
   UINT32 size;
   int res;
   
-  size = MIN(self->buffer->end - self->buffer->start,
-	     self->buffer->block_size);
+  size = MIN(self->write_buffer->end - self->write_buffer->start,
+	     self->write_buffer->block_size);
   assert(size);
   
   res = write(fd->fd,
-	      self->buffer->buffer + self->buffer->start,
+	      self->write_buffer->buffer + self->write_buffer->start,
 	      size);
   if (!res)
     fatal("Closed?");
@@ -373,21 +536,23 @@ static void write_callback(struct lsh_fd *fd)
 	break;
       default:
 	werror("io.c: write failed, %z\n", STRERROR(errno));
-
+	EXCEPTION_RAISE(fd->e,
+			make_io_exception(EXC_IO_WRITE, fd, errno, NULL));
 	close_fd(fd, CLOSE_WRITE_FAILED);
 	
 	break;
       }
   else
-    write_buffer_consume(self->buffer, res);
+    write_buffer_consume(self->write_buffer, res);
 }  
 
-static void listen_callback(struct lsh_fd *fd)
+static void
+do_listen_callback(struct io_read_callback *s UNUSED,
+		   struct lsh_fd *fd)
 {
   CAST(listen_fd, self, fd);
   struct sockaddr_in peer;
   size_t addr_len = sizeof(peer);
-  int res;
   int conn;
 
   conn = accept(fd->fd,
@@ -397,19 +562,14 @@ static void listen_callback(struct lsh_fd *fd)
       werror("io.c: accept() failed, %z", STRERROR(errno));
       return;
     }
-  res = FD_LISTEN_CALLBACK(self->callback, conn, 
-			   sockaddr2info(addr_len,
-					 (struct sockaddr *) &peer));
-  if (LSH_ACTIONP(res))
-    {
-      werror("Strange: Accepted a connection, "
-	     "but failed before writing anything.\n");
-      close_fd(fd, (LSH_FAILUREP(res)
-		    ? CLOSE_PROTOCOL_FAILURE
-		    : CLOSE_EOF));
-    }
+  FD_LISTEN_CALLBACK(self->callback, conn, 
+		     sockaddr2info(addr_len,
+				   (struct sockaddr *) &peer));
 }
 
+static struct io_read_callback listen_callback =
+{ STATIC_HEADER, do_listen_callback };
+
 static void connect_callback(struct lsh_fd *fd)
 {
   CAST(connect_fd, self, fd);
@@ -422,22 +582,15 @@ static void connect_callback(struct lsh_fd *fd)
       || socket_error)
     {
       debug("io.c: connect_callback: Connect failed.\n");
-      (void) FD_CALLBACK(self->callback, -1);
+      EXCEPTION_RAISE(fd->e,
+		      make_io_exception(EXC_CONNECT, fd, 0, "connect() failed."));
     }
   else
     {
-      int res = FD_CALLBACK(self->callback, fd->fd);
-      
-      if (LSH_ACTIONP(res))
-	{
-	  werror("Strange: Connected, "
-		 "but failed before writing anything.\n");
-	}
-      else
-	{ /* Everything seems fine. */
-	  /* To avoid actually closing the fd */
-	  fd->fd = -1;
-	}
+      FD_CALLBACK(self->callback, fd->fd);
+
+      /* To avoid actually closing the fd */
+      fd->fd = -1;
     }
   kill_fd(fd);
 }
@@ -467,7 +620,7 @@ void init_backend(struct io_backend *b)
 #endif
 }
 
-/* This function is called if a connection this file somehow dependent
+/* This function is called if a connection this file somehow depends
  * on disappears. For instance, the connection may have spawned a
  * child process, and this file may be the stdin of that process. */
 
@@ -476,18 +629,20 @@ static void do_kill_fd(struct resource *r)
 {
   CAST_SUBTYPE(lsh_fd, fd, r);
 
-  /* FIXME: It could make sense to you a separate close reason for
-   * killed files. For now, using the zero reason will supress calling
-   * of any close callbacks. */
+  /* NOTE: We use the zero close reason for files killed this way.
+   * Close callbacks are still called, but they should probably not do
+   * anything if reason == 0. */
   if (r->alive)
     close_fd(fd, 0);
 }
 
 /* Initializes a file structure, and adds it to the backend's list. */
-static void init_file(struct io_backend *b, struct lsh_fd *f, int fd)
+static void init_file(struct io_backend *b, struct lsh_fd *f, int fd,
+		      struct exception_handler *e)
 {
-  /* assert(fd); */
   f->fd = fd;
+
+  f->e = e;
   f->close_reason = -1; /* Invalid reason */
   f->close_callback = NULL;
 
@@ -507,29 +662,78 @@ static void init_file(struct io_backend *b, struct lsh_fd *f, int fd)
   b->files = f;
 }
 
+
 /* Blocking read from a file descriptor (i.e. don't use the backend).
  * The fd should *not* be in non-blocking mode. */
 
+/* FIXME: How to do this when moving from return codes to exceptions? */
+
+/* FIXME: The entire blocking_read mechanism should be replaced by
+ * ordinary commands and non-blocking i/o command. Right now, it is
+ * used to read key-files, so that change probably has to wait until
+ * the parser is rewritten. */
+
+#if 0
+/* ;; GABA:
+   (class
+     (name blocking_read_exception_handler)
+     (super exception_handler)
+     (vars
+       (flag . int)))
+*/
+
+static void
+do_blocking_read_handler(struct exception_handler *s,
+			 const struct exception *e UNUSED)
+{
+  CAST(blocking_read_exception_handler, self, s);
+  self->flag = 1;
+}
+#endif
+
+#define BLOCKING_READ_SIZE 4096
+
 int blocking_read(int fd, struct read_handler *handler)
 {
-  struct fd_read r =
-  { { STACK_HEADER, do_read }, fd };
-
+#if 0
+  struct blocking_read_exception_handler exc_handler =
+  {
+    { STACK_HEADER, do_blocking_read_handler },
+    0
+  };
+#endif
+  
+  char *buffer = alloca(BLOCKING_READ_SIZE);
+  
   for (;;)
     {
-      int res = READ_HANDLER(handler,
-			     &r.super);
-
-      assert(!(res & (LSH_HOLD | LSH_KILL_OTHERS)));
-
-      if (res & (LSH_CLOSE | LSH_DIE))
+      int res = read(fd, buffer, BLOCKING_READ_SIZE);
+      if (res < 0)
+	switch(errno)
+	  {
+	  case EINTR:
+	    break;
+	  case EWOULDBLOCK:
+	    fatal("blocking_read: Unexpected EWOULDBLOCK! fd in non-blocking mode?\n");
+	  default:
+	    werror("blocking_read: read() failed (errno = %i): %z\n",
+		   errno, strerror(errno));
+	    return 0;
+	  }
+      else if (!res)
+	return 1;
+      else
 	{
-	  close(fd);
-	  return res;
+	  UINT32 got = res;
+	  UINT32 done = 0;
+
+	  while (handler
+		 && (done < got))
+	    done += READ_HANDLER(handler, got - done, buffer + done);
 	}
-      if (res & LSH_FAIL)
-	werror("blocking_read: Ignoring error %i\n", res);
     }
+  close(fd);
+  return !handler;
 }
 
 /* Converts a string port number or service name to a port number.
@@ -744,8 +948,11 @@ int address_info2sockaddr_in(struct sockaddr_in *sin,
     return tcp_addr(sin, 0, NULL, a->port);
 }
 
+/* These functions are used by werror() and friends */
+
 /* For fd:s in blocking mode. */
-int write_raw(int fd, UINT32 length, UINT8 *data)
+void write_raw(int fd, UINT32 length, UINT8 *data,
+	       struct exception_handler *e)
 {
   while(length)
     {
@@ -758,16 +965,18 @@ int write_raw(int fd, UINT32 length, UINT8 *data)
 	  case EAGAIN:
 	    continue;
 	  default:
-	    return 0;
+	    EXCEPTION_RAISE(e, make_io_exception(EXC_IO_BLOCKING_WRITE,
+						 NULL, errno, NULL));
+	    return;
 	  }
       
       length -= written;
       data += written;
     }
-  return 1;
 }
 
-int write_raw_with_poll(int fd, UINT32 length, UINT8 *data)
+void write_raw_with_poll(int fd, UINT32 length, UINT8 *data,
+			 struct exception_handler *e)
 {
   while(length)
     {
@@ -787,7 +996,8 @@ int write_raw_with_poll(int fd, UINT32 length, UINT8 *data)
 	  case EAGAIN:
 	    continue;
 	  default:
-	    return 0;
+	    EXCEPTION_RAISE(e, make_io_exception(EXC_IO_BLOCKING_WRITE,
+						 NULL, errno, NULL));
 	  }
       
       written = write(fd, data, length);
@@ -799,13 +1009,13 @@ int write_raw_with_poll(int fd, UINT32 length, UINT8 *data)
 	  case EAGAIN:
 	    continue;
 	  default:
-	    return 0;
+	    EXCEPTION_RAISE(e, make_io_exception(EXC_IO_BLOCKING_WRITE,
+						 NULL, errno, NULL));
 	  }
       
       length -= written;
       data += written;
     }
-  return 1;
 }
 
 void io_set_nonblocking(int fd)
@@ -830,7 +1040,7 @@ void io_init_fd(int fd)
   io_set_close_on_exec(fd);
 }
 
-/* Some code is taken from bellman's tcputils. */
+/* Some code is taken from Thomas Bellman's tcputils. */
 struct connect_fd *io_connect(struct io_backend *b,
 			      struct sockaddr_in *remote,
 			      struct sockaddr_in *local,
@@ -865,7 +1075,8 @@ struct connect_fd *io_connect(struct io_backend *b,
   {
     NEW(connect_fd, fd);
 
-    init_file(b, &fd->super, s);
+    /* FIXME: What handler to use? */
+    init_file(b, &fd->super, s, &default_exception_handler);
 
     fd->super.want_write = 1;
     fd->super.write = connect_callback;
@@ -908,49 +1119,56 @@ struct listen_fd *io_listen(struct io_backend *b,
   {
     NEW(listen_fd, fd);
 
-    init_file(b, &fd->super, s);
+    /* FIXME: What handler to use? */
+    init_file(b, &fd->super, s, &default_exception_handler);
     
     fd->super.want_read = 1;
-    fd->super.read = listen_callback;
+    fd->super.read = &listen_callback;
     fd->callback = callback;
     
     return fd;
   }
 }
 
+/* FIXME: Closing the write buffer should perhaps be done earlier,
+ * from kill_fd(). */
 static void really_close(struct lsh_fd *fd)
 {
   CAST(io_fd, self, fd);
 
-  assert(self->buffer);
+  assert(self->write_buffer);
 
-  write_buffer_close(self->buffer);
+  write_buffer_close(self->write_buffer);
 }
 
 static void prepare_write(struct lsh_fd *fd)
 {
   CAST(io_fd, self, fd);
 
-  assert(self->buffer);
+  assert(self->write_buffer);
 
-  if (! (fd->want_write = write_buffer_pre_write(self->buffer))
-      && self->buffer->closed)
+  if (! (fd->want_write = write_buffer_pre_write(self->write_buffer))
+      && self->write_buffer->closed)
     close_fd(fd, CLOSE_EOF);
 }
 
+
+/* Constructors */
+
 struct io_fd *make_io_fd(struct io_backend *b,
-			 int fd)
+			 int fd,
+			 struct exception_handler *e)
 {
   NEW(io_fd, f);
 
   io_init_fd(fd);
-  init_file(b, &f->super, fd);
+  init_file(b, &f->super, fd, e);
 
   return f;
 }
 
 struct io_fd *io_read_write(struct io_fd *fd,
-			    struct read_handler *handler,
+			    struct io_read_callback *read,
 			    UINT32 block_size,
 			    struct close_callback *close_callback)
 {
@@ -960,14 +1178,17 @@ struct io_fd *io_read_write(struct io_fd *fd,
 	fd->super.fd);
   
   /* Reading */
-  fd->super.read = read_callback;
-  fd->super.want_read = !!handler;
-  fd->handler = handler;
-
+  fd->super.read = read;
+  fd->super.want_read = !!read;
+#if 0
+  fd->handler = handler; 
+  fd->read_buffer = make_fd_read_buffer(READ_BUFFER_SIZE); 
+#endif
+  
   /* Writing */
   fd->super.prepare = prepare_write;
   fd->super.write = write_callback;
-  fd->buffer = buffer;
+  fd->write_buffer = buffer;
 
   /* Closing */
   fd->super.really_close = really_close;
@@ -977,16 +1198,19 @@ struct io_fd *io_read_write(struct io_fd *fd,
 }
 
 struct io_fd *io_read(struct io_fd *fd,
-		      struct read_handler *handler,
+		      struct io_read_callback *read,
 		      struct close_callback *close_callback)
 {
   debug("io.c: Preparing fd %i for reading\n", fd->super.fd);
   
   /* Reading */
-  fd->super.want_read = !!handler;
-  fd->super.read = read_callback;
+  fd->super.want_read = !!read;
+  fd->super.read = read;
+#if 0
   fd->handler = handler;
-
+  fd->read_buffer = make_fd_read_buffer(READ_BUFFER_SIZE);
+#endif
+  
   fd->super.close_callback = close_callback;
 
   return fd;
@@ -1003,8 +1227,8 @@ struct io_fd *io_write(struct io_fd *fd,
   /* Writing */
   fd->super.prepare = prepare_write;
   fd->super.write = write_callback;
-  fd->buffer = buffer;
-  
+  fd->write_buffer = buffer;
+
   fd->super.close_callback = close_callback;
 
   return fd;
@@ -1021,3 +1245,86 @@ void close_fd(struct lsh_fd *fd, int reason)
   fd->close_reason = reason;
   kill_fd(fd);
 }
+
+void close_fd_nicely(struct lsh_fd *fd, int reason)
+{
+  /* Don't attempt to read any further. */
+  /* FIXME: Is it safe to free the handler here? */
+
+  fd->want_read = 0;
+  fd->read = NULL;
+  
+  if (fd->really_close)
+    /* Mark the write_buffer as closed */
+    REALLY_CLOSE_FD(fd);
+  else
+    /* There's no data buffered for write. */
+    kill_fd(fd);
+
+  fd->close_reason = reason;
+}
+
+/* Responsible for handling the EXC_FINISH_READ exception. It should
+ * be a parent to the connection related exception handlers, as for
+ * instance the protocol error handler will raise the EXC_FINISH_READ
+ * exception. */
+/* GABA:
+   (class
+     (name exc_finish_read_handler)
+     (super exception_handler)
+     (vars
+       (fd object lsh_fd)))
+*/
+
+static void
+do_exc_finish_read_handler(struct exception_handler *s,
+			   const struct exception *e)
+{
+  CAST(exc_finish_read_handler, self, s);
+  switch(e->type)
+    {
+    case EXC_FINISH_READ:
+      /* FIXME: What to do about the reason argument? */
+      close_fd_nicely(self->fd, 0);
+      break;
+    case EXC_FINISH_IO:
+      close_fd(self->fd, 0);
+      break;
+    default:
+      EXCEPTION_RAISE(self->super.parent, e);
+    }
+}
+
+struct exception_handler *
+make_exc_finish_read_handler(struct lsh_fd *fd,
+			     struct exception_handler *parent)
+{
+  NEW(exc_finish_read_handler, self);
+  self->fd = fd;
+  self->super.parent = parent;
+  self->super.raise = do_exc_finish_read_handler;
+
+  return &self->super;
+}
+
+struct exception finish_read_exception =
+STATIC_EXCEPTION(EXC_FINISH_READ, "Finish i/o");
+
+struct exception *
+make_io_exception(UINT32 type, struct lsh_fd *fd, int error, const char *msg)
+{
+  NEW(io_exception, self);
+  assert(type & EXC_IO);
+  
+  self->super.type = type;
+
+  if (msg)
+    self->super.msg = msg;
+  else
+    self->super.msg = error ? strerror(error) : "Unknown i/o error";
+
+  self->error = error;
+  self->fd = fd;
+  
+  return &self->super;
+}
