@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: io.h,v 1.35 1999/06/10 06:32:45 nisse Exp $ */
+ * $Id: io.h,v 1.41 1999/08/22 21:24:11 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -36,16 +36,21 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 
+/* Forward declarations */
+
+struct lsh_fd;
+
 #define GABA_DECLARE
 #include "io.h.x"
 #undef GABA_DECLARE
 
+
 /* A closed function with a file descriptor as argument */
 /* GABA:
    (class
      (name fd_callback)
      (vars
-       (f indirect-method int "int fd")))
+       (f indirect-method void "int fd")))
 */
 
 #define FD_CALLBACK(c, fd) ((c)->f(&(c), (fd)))
@@ -53,7 +58,8 @@
 /* Close callbacks are called with a reason as argument. */
 
 /* End of file while reading.
- * Or when a closed write_buffer has been flushed successfully. */
+ * Or when a closed write_buffer has been flushed successfully.
+ * Or when poll() returns POLLHUP. */
 /* FIXME: Should we use separate codes for these two events? */
 #define CLOSE_EOF 1
 
@@ -75,6 +81,18 @@
 
 #define CLOSE_CALLBACK(c, r) ((c)->f((c), (r)))
 
+/* The fd read callback is a closure, in order to support different
+ * reading styles (buffered and consuming). */
+
+/* GABA:
+   (class
+     (name io_read_callback)
+     (vars
+       (read method void "struct lsh_fd *fd")))
+*/
+
+#define IO_READ_CALLBACK(c, fd) ((c)->read((c), (fd)))
+
 /* GABA:
    (class
      (name lsh_fd)
@@ -83,27 +101,35 @@
        (next object lsh_fd)
        (fd simple int)
 
+       ; Used for raising i/o- and eof-exceptions.
+       ; Also passed on to readers of the consuming type,
+       ; which seems kind of bogus.
+       (e object exception_handler)
+       
+       ;; FIXME: Can the close handlers be replaced by exceptions?
        ; User's close callback
        (close_reason simple int)
        (close_callback object close_callback)
 
        ; Called before poll
        (prepare method void)
-
+       
        (want_read simple int)
        ; Called if poll indicates that data can be read. 
-       (read method void)
+       ;;(read method void)
+			    
+       (read object io_read_callback)
 
        (want_write simple int)
        ; Called if poll indicates that data can be written.
        (write method void)
 
-       ; (close_now simple int)
-       (really_close method void)))
-*/
+       ; FIXME: really_close is an improper name. It is used to
+       ; mark the fd's write_buffer as closed.
+       (really_close method void))) */
 
 #define PREPARE_FD(fd) ((fd)->prepare((fd)))
-#define READ_FD(fd) ((fd)->read((fd)))
+#define READ_FD(fd) IO_READ_CALLBACK((fd)->read, (fd))
 #define WRITE_FD(fd) ((fd)->write((fd)))
 #define REALLY_CLOSE_FD(fd) ((fd)->really_close((fd)))
 
@@ -113,11 +139,48 @@
      (super lsh_fd)
      (vars
        ; Reading 
-       (handler object read_handler)
+       ;;(read_buffer object read_buffer)
+       ;; (handler object read_handler)
        ; Writing 
-       (buffer object write_buffer)))
+       (write_buffer object write_buffer)))
 */
 
+/* Used for read handlers like read_line and read_packet that
+ * processes a little data at a time, possibly replacing the handler
+ * and leaving some data for the new one. */
+
+/* GABA:
+   (class
+     (name io_buffered_read)
+     (super io_read_callback)
+     (vars
+       (buffer_size . UINT32)
+       (handler object read_handler)))
+*/
+
+struct io_read_callback *
+make_buffered_read(UINT32 buffer_size,
+		   struct read_handler *handler);
+
+/* Used for read handlers like read_data, that know how much data they
+ * can consume. */
+
+/* GABA:
+   (class
+     (name io_consuming_read)
+     (super io_read_callback)
+     (vars
+       (query method UINT32)
+       ; Returns the maximum number of octets that
+       ; can be consumed immediately.
+       (consumer object abstract_write)))
+*/
+
+#define READ_QUERY(r) ((r)->query((r)))
+
+void init_consuming_read(struct io_consuming_read *self,
+			 struct abstract_write *consumer);
+
 /* Passed to the listen callback, and to other functions and commands
  * dealing with addresses. */
 /* GABA:
@@ -134,10 +197,13 @@
    (class
      (name fd_listen_callback)
      (vars
-       (f method int int "struct address_info *")))
+       (f method void int "struct address_info *")))
 */
 #define FD_LISTEN_CALLBACK(c, fd, a) ((c)->f((c), (fd), (a)))
 
+/* FIXME: Get rid of this class, and store the user callback inside
+ * accept from the io_read_callback. */
+
 /* GABA:
    (class
      (name listen_fd)
@@ -176,6 +242,28 @@ struct callout
        ))
 */
 
+/* I/O exceptions */
+/* GABA:
+   (class
+     (name io_exception)
+     (super exception)
+     (vars
+       ;; NULL if no fd was involved
+       (fd object lsh_fd)
+       ;; errno code, or zero if not available
+       (error . int))))
+*/
+
+/* If msg is NULL, it is derived from errno */
+struct exception *
+make_io_exception(UINT32 type, struct lsh_fd *fd, int error, const char *msg);
+
+/* Used in cases where the fd and errno are not available */
+#define STATIC_IO_EXCEPTION(type, name) \
+{ { STATIC_HEADER, (type), (name) }, NULL, 0}
+
+extern struct exception finish_read_exception;
+
 void init_backend(struct io_backend *b);
 
 int io_iter(struct io_backend *b);
@@ -207,15 +295,20 @@ struct address_info *sockaddr2info(size_t addr_len UNUSED,
 int address_info2sockaddr_in(struct sockaddr_in *sin,
 			     struct address_info *a);
 
-int write_raw(int fd, UINT32 length, UINT8 *data);
-int write_raw_with_poll(int fd, UINT32 length, UINT8 *data);
+void write_raw(int fd, UINT32 length, UINT8 *data, struct exception_handler *e);
+void write_raw_with_poll(int fd, UINT32 length, UINT8 *data, struct exception_handler *e);
 
 void io_set_nonblocking(int fd);
 void io_set_close_on_exec(int fd);
 void io_init_fd(int fd);
 
 struct io_fd *make_io_fd(struct io_backend *b,
-			 int fd);
+			 int fd,
+			 struct exception_handler *e);
+
+struct exception_handler *
+make_exc_finish_read_handler(struct lsh_fd *fd,
+			     struct exception_handler *parent);
 
 struct connect_fd *io_connect(struct io_backend *b,
 			      struct sockaddr_in *remote,
@@ -227,12 +320,12 @@ struct listen_fd *io_listen(struct io_backend *b,
 			    struct fd_listen_callback *callback);
 
 struct io_fd *io_read_write(struct io_fd *fd,
-			    struct read_handler *read_callback,
+			    struct io_read_callback *read,
 			    UINT32 block_size,
 			    struct close_callback *close_callback);
 
 struct io_fd *io_read(struct io_fd *fd,
-		      struct read_handler *read_callback,
+		      struct io_read_callback *read,
 		      struct close_callback *close_callback);
 
 struct io_fd *io_write(struct io_fd *fd,
@@ -244,5 +337,9 @@ void kill_fd(struct lsh_fd *fd);
 
 void close_fd(struct lsh_fd *fd, int reason);
 
+/* Stop reading from the fd, and close it as soon as the buffer
+ * is completely written. */
+void close_fd_nicely(struct lsh_fd *fd, int reason);
+
 
 #endif /* LSH_IO_H_INCLUDED */
