@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.62 1999/06/30 21:03:37 nisse Exp $ */
+ * $Id: lsh.c,v 1.67 1999/08/28 19:57:14 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -132,8 +132,8 @@ static struct lookup_verifier *make_fake_host_db(struct signature_algorithm *a)
        (progn "&progn_command.super.super")
        ;; FIXME: Use some function that also displays an error message
        ;; if connect() fails.
-       (init_connection "&connection_service.super")
-       (die_on_null "&command_die_on_null.super"))
+       (init_connection "&connection_service.super"))
+       ;; (die_on_null "&command_die_on_null.super"))
      (params
        (connect object command)
        (handshake object command)
@@ -144,7 +144,7 @@ static struct lookup_verifier *make_fake_host_db(struct signature_algorithm *a)
      (expr (lambda (port)
              ((progn requests) (init_connection
 	         (login (userauth_service
-	           (handshake (die_on_null (connect port))))))))))
+	           (handshake (connect port)))))))))
 */
 
 /* GABA:
@@ -208,6 +208,53 @@ static int parse_forward_arg(char *arg,
   return 1;
 }
 
+/* GABA:
+   (class
+     (name lsh_default_handler)
+     (super exception_handler)
+     (vars
+       (status . "int *")))
+*/
+
+static void
+do_lsh_default_handler(struct exception_handler *s,
+		       const struct exception *e)
+{
+  CAST(lsh_default_handler, self, s);
+
+  switch(e->type)
+    {
+    case EXC_CONNECT:
+    case EXC_RESOLVE:
+      werror("lsh: Connection failed: %z\n", e->msg);
+      break;
+    case EXC_IO_READ:
+    case EXC_IO_WRITE:
+      werror("lsh: Connection broken: %z\n", e->msg);
+      break;
+    case EXC_AUTH:
+      werror("lsh: Host authentication failed: %z\n", e->msg);
+      break;
+    case EXC_SERVICE:
+      werror("lsh: Service denied: %z\n", e->msg);
+      break;
+    default:
+      EXCEPTION_RAISE(self->super.parent, e);
+    }
+  *self->status = EXIT_FAILURE;
+}
+
+static struct exception_handler *
+make_lsh_default_handler(int *status, struct exception_handler *parent)
+{
+  NEW(lsh_default_handler, self);
+  self->super.parent = parent;
+  self->super.raise = do_lsh_default_handler;
+  self->status = status;
+
+  return &self->super;
+}
+
 /* Window size for the session channel */
 #define WINDOW_SIZE (SSH_MAX_PACKET << 3)
 
@@ -252,6 +299,12 @@ int main(int argc, char **argv)
   
   /* int in, out, err; */
 
+  /* FIXME: A single exception handler everywhere seems a little to
+   * crude. */
+  struct exception_handler *handler
+    = make_lsh_default_handler(&lsh_exit_code, &default_exception_handler);
+
+
   NEW(io_backend, backend);
 
   /* For filtering messages. Could perhaps also be used when converting
@@ -478,10 +531,11 @@ int main(int argc, char **argv)
 	(&actions,
 	 make_start_session
 	 (make_open_session_command(make_client_session
-				    (io_read(make_io_fd(backend, in), NULL, NULL),
-				     io_write(make_io_fd(backend, out),
+				    (io_read(make_io_fd(backend, in, handler),
+					     NULL, NULL),
+				     io_write(make_io_fd(backend, out, handler),
 					      BLOCK_SIZE, NULL),
-				     io_write(make_io_fd(backend, err),
+				     io_write(make_io_fd(backend, err, handler),
 					      BLOCK_SIZE, NULL),
 				     WINDOW_SIZE,
 				     &lsh_exit_code)),
@@ -531,16 +585,13 @@ int main(int argc, char **argv)
 			      queue_to_list(&actions));
 
 	CAST_SUBTYPE(command, client_connect, o);
-	int res = COMMAND_CALL(client_connect, remote, NULL);
 
+	COMMAND_CALL(client_connect, remote, &discard_continuation,
+		     handler);
+	
 	/* We can free the queue nodes now */
 	KILL_OBJECT_QUEUE(&actions);
 
-	if (res)
-	  {
-	    werror("lsh.c: connection failed, res = %i\n", res);
-	    return 17;
-	  }
       }
       io_run(backend);
 
