@@ -2,7 +2,7 @@
  *
  * Reads a (private) key on stdin, and saves it a private and a public file.
  *
- * $Id: lsh_writekey.c,v 1.8 1999/05/12 21:35:20 nisse Exp $ */
+ * $Id: lsh_writekey.c,v 1.10 1999/08/28 19:57:16 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -26,6 +26,7 @@
 #include "blocking_write.h"
 #include "format.h"
 #include "io.h"
+#include "read_scan.h"
 #include "sexp.h"
 #include "werror.h"
 #include "xalloc.h"
@@ -43,6 +44,19 @@
 
 #include "lsh_writekey.c.x"
 
+static void
+do_lsh_writekey_handler(struct exception_handler *s UNUSED,
+			const struct exception *e)
+{
+  werror("lsh_writekey: %z\n", e->msg);
+
+  /* FIXME: It would be better to set the exit_success variable. */
+  exit(EXIT_FAILURE);
+}
+
+static struct exception_handler exc_handler =
+STATIC_EXCEPTION_HANDLER(do_lsh_writekey_handler, NULL);
+
 /* FIXME: Should support encryption of the private key. */
 
 static struct sexp *dsa_private2public(struct sexp_iterator *i)
@@ -96,7 +110,8 @@ static struct sexp *dsa_private2public(struct sexp_iterator *i)
        (private_file . "char *")))
 */
 
-static int do_write_key(struct sexp_handler *h, struct sexp *private)
+static int
+do_write_key(struct sexp_handler *h, struct sexp *private)
 {
   CAST(write_key, closure, h);
   struct sexp_iterator *i;
@@ -141,24 +156,17 @@ static int do_write_key(struct sexp_handler *h, struct sexp *private)
       return LSH_FAIL | LSH_DIE;
     }
 
-  if (LSH_FAILUREP(A_WRITE(make_blocking_write(public_fd, 0),
-			   sexp_format(public, SEXP_TRANSPORT, 0))))
-    {
-      werror("lsh_writekey: Writing to %z failed (errno = %i): %z\n",
-	     closure->public_file, errno, STRERROR(errno));
-      return LSH_FAIL | LSH_DIE;
-    }
-  
-  if (LSH_FAILUREP(A_WRITE(make_blocking_write(private_fd, 0),
-			   sexp_format(private, SEXP_CANONICAL, 0))))
-    {
-      werror("lsh_writekey: Writing to %z failed (errno = %i): %z\n",
-	     closure->private_file, errno, STRERROR(errno));
-      return LSH_FAIL | LSH_DIE;
-    }
+  A_WRITE(make_blocking_write(public_fd, 0),
+	  sexp_format(public, SEXP_TRANSPORT, 0),
+	  &exc_handler);
 
+  A_WRITE(make_blocking_write(private_fd, 0),
+	  sexp_format(private, SEXP_CANONICAL, 0),
+	  &exc_handler);
+  
   *closure->status = EXIT_SUCCESS;
-  return LSH_OK | LSH_DIE;
+
+  return LSH_OK | LSH_CLOSE;
 }
 
 #define BLOCK_SIZE 2000
@@ -230,8 +238,9 @@ int main(int argc UNUSED, char **argv UNUSED)
   handler->public_file = public;
   handler->private_file = private;
   
-  io_read(make_io_fd(backend, STDIN_FILENO),
-	  make_read_sexp(&handler->super, BLOCK_SIZE, SEXP_TRANSPORT, 0),
+  io_read(make_io_fd(backend, STDIN_FILENO, &exc_handler),
+	  make_buffered_read(BLOCK_SIZE,
+			     make_read_sexp(&handler->super, SEXP_TRANSPORT, 0)),
 	  NULL);
 
   io_run(backend);
