@@ -2,7 +2,7 @@
  *
  * main server program.
  *
- * $Id: lshd.c,v 1.52 1999/06/11 23:46:55 nisse Exp $ */
+ * $Id: lshd.c,v 1.60 1999/08/28 19:57:17 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -27,6 +27,7 @@
 #include "alist.h"
 #include "atoms.h"
 #include "channel.h"
+#include "channel_commands.h"
 #include "charset.h"
 #include "connection_commands.h"
 #include "crypto.h"
@@ -36,17 +37,20 @@
 #include "server_password.h"
 #include "server_session.h"
 #include "randomness.h"
+#include "read_scan.h"
 #include "reaper.h"
 #include "server.h"
 #include "server_keyexchange.h"
 #include "sexp.h"
 #include "ssh.h"
+#include "tcpforward_commands.h"
 #include "userauth.h"
 #include "werror.h"
 #include "xalloc.h"
 #include "compress.h"
 #include "server_pty.h"
 #include "tcpforward.h"
+#include "tcpforward_commands.h"
 
 #include "getopt.h"
 
@@ -66,9 +70,6 @@
 #include <unistd.h>
 #endif
 
-/* FIXME: Add configure and runtime options instead */
-#define WITH_TCPFORWARD_SUPPORT 1
-
 /* Block size for stdout and stderr buffers */
 #define BLOCK_SIZE 32768
 
@@ -85,6 +86,9 @@ void usage(void)
 	 " -z,  --compression[=ALGORITHM]\n"
 	 "      --mac=ALGORITHM\n"
 	 " -q,  --quiet\n"
+#if WITH_TCP_FORWARD
+	 "      --no-forward\n"
+#endif
 #if WITH_SSH1_FALLBACK
          "      --ssh1-fallback=SSHD\n"
 #endif
@@ -113,7 +117,6 @@ static int do_read_key(struct sexp_handler *h, struct sexp *private)
   struct sexp_iterator *i;
   struct sexp *e;
   mpz_t p, q, g, y, x;
-
   int res;
   
   if (!sexp_check_type(private, "private-key", &i))
@@ -225,32 +228,15 @@ static int read_host_key(const char *name,
       handler->keys = keys;
       
       res = blocking_read(fd, make_read_sexp(&handler->super,
-					     2000, SEXP_TRANSPORT, 0));
+					     SEXP_TRANSPORT, 0));
       close(fd);
 
       KILL(handler);
       
-      if (LSH_FAILUREP(res))
-	{
-	  werror("lshd: Invalid host key.\n");
-	  return 0;
-	}
-
       return 1;
     }
 }
 
-/* Invoked when the client requests the userauth service. */
-/* GABA:
-   (expr
-     (name lshd_services)
-     (params 
-       (userauth object command) )
-     (expr
-       (lambda (connection)
-         ((userauth connection) connection))))
-*/
-
 /* GABA:
    (expr
      (name lshd_listen)
@@ -264,6 +250,35 @@ static int read_host_key(const char *name,
              (services (handshake (log (listen port)))))))
 */
 
+/* Invoked when the client requests the userauth service. */
+/* GABA:
+   (expr
+     (name lshd_services)
+     (params 
+       (userauth object command))
+     (expr
+       (lambda (connection)
+         ((userauth connection) connection))))
+*/
+
+/* Invoked when starting the ssh-connection service */
+/* GABA:
+   (expr
+     (name lshd_connection_service)
+     (globals
+       (progn "&progn_command.super.super")
+       (init "&connection_service.super"))
+     (params
+       (login object command)     
+       (hooks object object_list))
+     (expr
+       (lambda (user connection)
+         ((progn hooks) (login user
+	                       ; We have to initialize the connection
+			       ; before logging in.
+	                       (init connection))))))
+*/
+
 int main(int argc, char **argv)
 {
   char *host = NULL;  /* Interface to bind */
@@ -274,6 +289,9 @@ int main(int argc, char **argv)
 #if WITH_SSH1_FALLBACK
   char *sshd1 = NULL;
 #endif
+#if WITH_TCP_FORWARD
+  int forward_flag = 1;
+#endif
   
   struct alist *keys;
   
@@ -310,7 +328,7 @@ int main(int argc, char **argv)
 
   for (;;)
     {
-      static struct option options[] =
+      struct option options[] =
       {
 	{ "verbose", no_argument, NULL, 'v' },
 	{ "quiet", no_argument, NULL, 'q' },
@@ -320,6 +338,9 @@ int main(int argc, char **argv)
 	{ "compression", optional_argument, NULL, 'z'},
 	{ "mac", required_argument, NULL, 'm' },
 	{ "hostkey", required_argument, NULL, 'h' },
+#if WITH_TCP_FORWARD
+	{ "no-forward", no_argument, &forward_flag, 0 },
+#endif
 #if WITH_SSH1_FALLBACK
 	{ "ssh1-fallback", optional_argument, NULL, OPT_SSH1_FALLBACK},
 #endif
@@ -443,60 +464,74 @@ int main(int argc, char **argv)
 			  make_int_list(0, -1));
   
   {
-    struct lsh_object *o = lshd_listen
-      (make_simple_listen(backend, NULL),
-       make_handshake_command(CONNECTION_SERVER,
-			      "lsh - a free ssh",
-			      SSH_MAX_PACKET,
-			      r,
-			      algorithms,
-			      make_kexinit,
+    /* Commands to be invoked on the connection */
+    struct object_list *connection_hooks;
+
+#if WITH_TCP_FORWARD
+    if (forward_flag)
+      connection_hooks = make_object_list
+	(3,
+	 make_tcpip_forward_hook(backend),
+	 make_install_fix_global_request_handler
+	 (ATOM_CANCEL_TCPIP_FORWARD, &tcpip_cancel_forward),
+	 make_direct_tcpip_hook(backend),
+	 -1);
+    else
+#endif
+      connection_hooks = make_object_list(0, -1);
+    {
+      struct lsh_object *o = lshd_listen
+	(make_simple_listen(backend, NULL),
+	 make_handshake_command(CONNECTION_SERVER,
+				"lsh - a free ssh",
+				SSH_MAX_PACKET,
+				r,
+				algorithms,
+				make_kexinit,
 #if WITH_SSH1_FALLBACK
-			      sshd1 ? make_ssh1_fallback (sshd1) :
+				sshd1 ? make_ssh1_fallback (sshd1) :
 #endif /* WITH_SSH1_FALLBACK */
-			      NULL),
-       make_offer_service
-       (make_alist
-	(1, ATOM_SSH_USERAUTH,
-	 lshd_services(make_userauth_service
-		       (make_int_list(1, ATOM_PASSWORD, -1),
-			make_alist(1, ATOM_PASSWORD,
-				   &unix_userauth.super, -1),
-			make_alist(1, ATOM_SSH_CONNECTION,
-				   make_server_connection_service
-				   (make_alist
-				    (0
-#if WITH_TCPFORWARD_SUPPORT
-				     +2,
-				     ATOM_TCPIP_FORWARD, make_tcpip_forward_request(backend),
-				     ATOM_CANCEL_TCPIP_FORWARD, make_cancel_tcpip_forward_request()
-#endif /* WITH_TCPFORWARD_SUPPORT */
-				     ,-1),
-				    make_alist
-				    (1
+				NULL),
+	 make_offer_service
+	 (make_alist
+	  (1, ATOM_SSH_USERAUTH,
+	   lshd_services(make_userauth_service
+			 (make_int_list(1, ATOM_PASSWORD, -1),
+			  make_alist(1, ATOM_PASSWORD,
+				     &unix_userauth.super, -1),
+			  make_alist(1, ATOM_SSH_CONNECTION,
+				     lshd_connection_service
+				     (make_server_connection_service
+				      (make_alist
+				       (1
 #if WITH_PTY_SUPPORT
-				     +1, ATOM_PTY_REQ, make_pty_handler()
+					+1, ATOM_PTY_REQ, make_pty_handler()
 #endif /* WITH_PTY_SUPPORT */
-				     , ATOM_SHELL,
-				     make_shell_handler(backend,
-							reaper),
-				     -1),
-				    backend),
-				   -1))),
-	 -1)));
+					, ATOM_SHELL,
+					make_shell_handler(backend,
+							   reaper),
+					-1),
+				       backend),
+				      connection_hooks),
+				     -1))),
+	   -1)));
     
-    CAST_SUBTYPE(command, server_listen, o);
+      CAST_SUBTYPE(command, server_listen, o);
     
-    int res = COMMAND_CALL(server_listen, local, NULL);
-    if (res)
-      {
-	if (res & LSH_COMMAND_FAILED)
-	    werror("lshd: Failed to bind port. (errno = %d) %z\n",
+      COMMAND_CALL(server_listen, local,
+		   &discard_continuation, &default_exception_handler);
+#if 0
+      if (res)
+	{
+	  if (res & LSH_COMMAND_FAILED)
+	    werror("lshd: Failed to bind port. (errno = %i) %z\n",
 		   errno, STRERROR(errno));
-	else
-	  werror("lshd: Unexpected failure from listen: %d\n", res);
-	return EXIT_FAILURE;
-      }
+	  else
+	    werror("lshd: Unexpected failure from listen: %i\n", res);
+	  return EXIT_FAILURE;
+	}
+#endif
+    }
   }
   
   reaper_run(reaper, backend);
