@@ -2,7 +2,7 @@
  *
  * A read handler for application data.
  *
- * $Id: read_data.h,v 1.7 1999/02/25 00:34:45 nisse Exp $ */
+ * $Id: read_data.h,v 1.9 1999/08/22 21:24:19 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -28,8 +28,12 @@
 
 #include "channel.h"
 
-struct read_handler *make_read_data(struct ssh_channel *channel,
-				    struct abstract_write *write);
+struct io_read_callback *make_read_data(struct ssh_channel *channel,
+					struct abstract_write *write);
+
+struct exception_handler *
+make_exc_read_eof_channel_handler(struct ssh_channel *channel,
+				  struct exception_handler *e);
 
 #endif /* LSH_READ_DATA_H_INCLUDED */
 
