@@ -1,6 +1,6 @@
 /* server.c
  *
- * $Id: server.c,v 1.55 1999/04/11 21:18:07 nisse Exp $ */
+ * $Id: server.c,v 1.58 1999/08/29 17:58:25 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -54,13 +54,15 @@
      (super packet_handler)
      (vars
        (services object alist)
-       (c object command_continuation)))
+       (c object command_continuation)
+       (e object exception_handler)))
 */
 
      
-static int do_service_request(struct packet_handler *c,
-			      struct ssh_connection *connection,
-			      struct lsh_string *packet)
+static void
+do_service_request(struct packet_handler *c,
+		   struct ssh_connection *connection,
+		   struct lsh_string *packet)
 {
   CAST(service_handler, closure, c);
 
@@ -82,39 +84,42 @@ static int do_service_request(struct packet_handler *c,
 	  CAST_SUBTYPE(command, service, ALIST_GET(closure->services, name));
 	  if (service)
 	    {
-	      int res;
-	      
 	      /* Don't accept any further service requests */
 	      connection->dispatch[SSH_MSG_SERVICE_REQUEST]
 		= connection->fail;
 
 	      /* Start service */
-	      res = A_WRITE(connection->write, format_service_accept(name));
-	      if (LSH_CLOSEDP(res))
-		return res;
-	      return res | COMMAND_CALL(service, connection, closure->c);
+	      C_WRITE(connection, format_service_accept(name));
+	      COMMAND_CALL(service, connection,
+			   closure->c, closure->e);
+	      return;
 	    }
 	}
-      return (LSH_FAIL | LSH_CLOSE)
-	| A_WRITE(connection->write,
-		  format_disconnect(SSH_DISCONNECT_SERVICE_NOT_AVAILABLE,
-				    "Service not available.", ""));
-    }
-
-  lsh_string_free(packet);
-  return LSH_FAIL | LSH_DIE;
-}
+      EXCEPTION_RAISE(connection->e,
+		      make_protocol_exception(SSH_DISCONNECT_SERVICE_NOT_AVAILABLE, NULL));
       
+    }
+  else
+    {
+      lsh_string_free(packet);
+      EXCEPTION_RAISE(connection->e,
+		      make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
+					      "Invalid SERVICE_REQUEST message"));
+    }
+}
+
 static struct packet_handler *
 make_service_request_handler(struct alist *services,
-			     struct command_continuation *c)
+			     struct command_continuation *c,
+			     struct exception_handler *e)
 {
   NEW(service_handler, self);
 
   self->super.handler = do_service_request;
   self->services = services;
   self->c = c;
-
+  self->e = e;
+  
   return &self->super;
 }
 
@@ -127,17 +132,17 @@ make_service_request_handler(struct alist *services,
        (services object alist)))
 */
 
-static int do_offer_service(struct command *s,
-			    struct lsh_object *x,
-			    struct command_continuation *c)
+static void
+do_offer_service(struct command *s,
+		 struct lsh_object *x,
+		 struct command_continuation *c,
+		 struct exception_handler *e)
 {
   CAST(offer_service, self, s);
   CAST(ssh_connection, connection, x);
 
   connection->dispatch[SSH_MSG_SERVICE_REQUEST]
-    = make_service_request_handler(self->services, c);
-
-  return LSH_OK | LSH_GOON;
+    = make_service_request_handler(self->services, c, e);
 }
 
 struct command *make_offer_service(struct alist *services)
